/**@name LIBRARY: pcConstant_declarations.ctl

@author:Herve Milcent(AB/CO)

Creation Date: 07-05-2007

Modification History:
 
  20/01/2015: JC
  - ENS-7242: Add Support for FGC3_PC
  
  07/03/2011: Herve
  - IS-482: FGCS MCS (POPS): use SUB_53 instead of SUB_52

  05/05/2010: Herve
  - FGC2_PS_MPC (Class 52) implementation
  
  - 25/08/2009: Herve
    new constant value for PC_FGC_STATE_IL, PC_FGC_STATE_RESET, PC_FGC_STATE_SB, 
  PC_FGC_STATE_POL_NEGATIVE and PC_FGC_STATE_POL_POSTIVE 
    
  - 16/06/2009: Herve & Sylvain: 
    new constant value for open loop

  - 06/08/2008: Herve & Sylvain: 
    new constant for open loop
    
	- 12/07/2007: Herve
		new constants for FGC offline
		
	- 08/06/207: Herve
		bug output address
		missing format I and V
		
	- 29/05/2007: Herve
		add ST_GTWDEV
	
version 1.0

Purpose: 
This library contains all the definition of the PC devices.

Usage: Public

PVSS manager usage: Ctrl, NV, NG

Constraints:
	. JCOP framework
	. PVSS version: 3.0
	. operating system: Linux and WXP but tested only on WXP.
	. distributed system: yes.
*/


// constant declaration
const string PC_DPNAME_PREFIX = "pc";

// constant for import of GW_FGC
const string PC_GW_FGC_DPTYPE = "GW_FGC";

// constant for import of GW_FGC Front-End
const unsigned UN_CONFIG_GW_FGC_LENGTH = 6;
const unsigned UN_CONFIG_GW_FGC_WFBUS = 4;
const unsigned UN_CONFIG_GW_FGC_DEVICE = 5;
const unsigned UN_CONFIG_GW_FGC_ARCHIVE_ACTIVE = 6;

// constant for GW_FGC system integrity
const string c_pc_FRONTEND_UNSYSTEMALARM = "_unSystemAlarm";
const string c_pcSystemIntegrity_GW_FGC = "GW_FGC_";
const string UN_SYSTEM_INTEGRITY_GW_FGC = "GW_FGC";
const string UN_SYSTEM_INTEGRITY_GW_FGC_check = "pcSystemIntegrity_GW_FGCCheck";
const string c_pcSystemIntegrity_GW_FGC_Time_callback = "pcSystemIntegrity_GW_FGCTimeCallback";
const string c_pcSystemIntegrity_GW_FGC_PmServerState_callback = "pcSystemIntegrity_GW_FGCPmServerStateCallback";
const int c_pcSystemIntegrity_defaultGW_FGCCheckingDelay = 10; // default value for the checking in sec.
const int C_GW_FGC_WAIT_BEFORE_GQ = 60;

// constant for import of FGC2_LHC_PC
const unsigned PC_CONFIG_FGC2_LHC_PC_LENGTH = 12;
const unsigned PC_CONFIG_FGC2_LHC_PC_I_FORMAT	= 1;
const unsigned PC_CONFIG_FGC2_LHC_PC_V_FORMAT	= 2;
const unsigned PC_CONFIG_FGC2_LHC_PC_MIN_I	= 3;
const unsigned PC_CONFIG_FGC2_LHC_PC_MAX_I	= 4;
const unsigned PC_CONFIG_FGC2_LHC_PC_MIN_V	= 5;
const unsigned PC_CONFIG_FGC2_LHC_PC_MAX_V	= 6;
const unsigned PC_CONFIG_FGC2_LHC_PC_DEADBAND	= 7;
const unsigned PC_CONFIG_FGC2_LHC_PC_DEADBANDTYPE	= 8;
const unsigned PC_CONFIG_FGC2_LHC_PC_ARCHIVEACTIVE	= 9;
const unsigned PC_CONFIG_FGC2_LHC_PC_DEVICENAME	= 10;
const unsigned PC_CONFIG_FGC2_LHC_PC_CMDADDRESS	= 11;
const unsigned PC_CONFIG_FGC2_LHC_PC_DIDT = 12;

// constant for the import database panel
const unsigned UN_CONFIG_GW_FGC_FGC2_LHC_PC_ARCHIVE_ACTIVE = 9;
const unsigned UN_CONFIG_GW_FGC_FGC2_LHC_PC_LENGTH = 12;

const string PC_CONFIG_I_UNIT = "A";
const string PC_CONFIG_V_UNIT = "V";

const string PC_CONFIG_3FORMAT = "###";
const string PC_CONFIG_5FORMAT = "#####";

const string PC_CONFIG_CMW_FGC2_LHC_PC_PROPERTY = "SUB_51:fgc.be";
const string PC_CONFIG_CMW_PSEUDO_FGC2_LHC_PC_PROPERTY = "SUB_51:pseudo.dev";
const string PC_CONFIG_CMW_GW_FGC_PROPERTY = "SUB_1:fgc.be";
const string PC_CONFIG_CMW_TAG = "value";
const int PC_CONFIG_FGC2_LHC_PC_INPUT_DPE = 19;
const int PC_CONFIG_FGC2_LHC_PC_OUTPUT_DPE = 3;

const string PC_STATE_PC_TEXT_LABELDP = "fgcSTATE_PC_TextLabel";
const string PC_STATE_OP_TEXT_LABELDP = "fgcSTATE_OP_TextLabel";
const int PC_TEXT_LABEL_TEXT = 1;
const int PC_TEXT_LABEL_COLOR = 2;

const string PC_FACEPLATE_BUTTON_ON = "On";
const string PC_FACEPLATE_BUTTON_OFF = "Off";
const string PC_FACEPLATE_BUTTON_IDLE = "Idle";
const string PC_FACEPLATE_BUTTON_RESET = "Reset";
const string PC_FACEPLATE_BUTTON_POLARITY = "Change Polarity";

const int PC_FACEPLATE_BUTTON_ONPOS = 1;
const int PC_FACEPLATE_BUTTON_OFFPOS = 2;
const int PC_FACEPLATE_BUTTON_SETVALUEPOS = 3;
const int PC_FACEPLATE_BUTTON_IDELPOS = 4;
const int PC_FACEPLATE_BUTTON_POLARITYPOS = 5;
const int PC_FACEPLATE_BUTTON_RESETPOS = 6;

const int PC_FGC_FLT_OFF = 0;
const int PC_FGC_OFF = 1;
const int PC_FGC_IDLE = 8;
const int PC_FGC_RUNNING = 11;
const int PC_FGC_ON_STANDBY = 7;

const string PC_FGC_STATE_IL = "IDLE";
const string PC_FGC_STATE_RESET = "OFF";
const string PC_FGC_STATE_SB = "ON_STANDBY";
const string PC_FGC_STATE_OFF = "OFF";
const string PC_FGC_STATE_POL_NEGATIVE = "NEGATIVE";
const string PC_FGC_STATE_POL_POSTIVE = "POSITIVE";
const string PC_FGC_SET_CURRENT = "NOW,current,,di/dt";
const string PC_FGC_SET_CURRENT_CURRENT = "current";
const string PC_FGC_SET_CURRENT_DIDT = "di/dt";

const string PC_FGC_SET_REF = "OPENLOOP,current";
const int PC_FGC_SET_RUN = 1;
const int PC_FGC_SET_OPEN_LOOP_DELAY = 1;

const string PC_WIDGET_TEXT_BAD_DEVICE = "F";

const int PC_TEXT_LABEL_FGC_OFFLINE = -1;
const int PC_TEXT_LABEL_FGC_UNKNOWN = -2;
const string PC_FGC_UNKNOWN_VALUE = "??";

const int PC_ONOFF_BUTTON_IMEAS_MAX = 450;

// constant for import of FGC2_PS_MPC
const unsigned PC_CONFIG_FGC2_PS_MPC_LENGTH = 14;
const unsigned PC_CONFIG_FGC2_PS_MPC_B_FORMAT	= 1;
const unsigned PC_CONFIG_FGC2_PS_MPC_V_FORMAT	= 2;
const unsigned PC_CONFIG_FGC2_PS_MPC_I_FORMAT	= 3;
const unsigned PC_CONFIG_FGC2_PS_MPC_MIN_B	= 4;
const unsigned PC_CONFIG_FGC2_PS_MPC_MAX_B	= 5;
const unsigned PC_CONFIG_FGC2_PS_MPC_MIN_V	= 6;
const unsigned PC_CONFIG_FGC2_PS_MPC_MAX_V	= 7;
const unsigned PC_CONFIG_FGC2_PS_MPC_MIN_I	= 8;
const unsigned PC_CONFIG_FGC2_PS_MPC_MAX_I	= 9;
const unsigned PC_CONFIG_FGC2_PS_MPC_DEADBAND	= 10;
const unsigned PC_CONFIG_FGC2_PS_MPC_DEADBANDTYPE	= 11;
const unsigned PC_CONFIG_FGC2_PS_MPC_ARCHIVEACTIVE	= 12;
const unsigned PC_CONFIG_FGC2_PS_MPC_DEVICENAME	= 13;
const unsigned PC_CONFIG_FGC2_PS_MPC_CMWOUTPUTADDRESS	= 14;

// constant for the import database panel
const unsigned UN_CONFIG_GW_FGC_FGC2_PS_MPC_ARCHIVE_ACTIVE = 12;
const unsigned UN_CONFIG_GW_FGC_FGC2_PS_MPC_LENGTH = 14;

const string PC_CONFIG_B_UNIT = "G";

const string PC_CONFIG_CMW_FGC2_PS_MPC_PROPERTY = "SUB_53:fgc.be";
const string PC_CONFIG_CMW_PSEUDO_FGC2_PS_MPC_PROPERTY = "SUB_53:pseudo.dev";

const int PC_CONFIG_FGC2_PS_MPC_INPUT_DPE = 20;
const int PC_CONFIG_FGC2_PS_MPC_OUTPUT_DPE = 2;

const string CONFIG_MODE_SYNC_NONE = "SYNC_NONE";
const string CONFIG_MODE_SYNC_DB = "SYNC_DB";
const string CONFIG_MODE_SYNC_FGC = "SYNC_FGC";
const string CONFIG_MODE_SYNC_DB_CAL = "SYNC_DB_CAL";
const string CONFIG_MODE_SYNC_FAILED = "SYNC_FAILED";

const int FGC2_PS_MPC_PAL_CHAN_DIVPAM_ID = 8;

// constant for FGC_PS_MPC device action
const string PC_FACEPLATE_BUTTON_PAL_CHAN_DIVPAM_ID = "PAL_CHAN_DIVPAM_ID";
const string PC_FACEPLATE_BUTTON_CONFIG_MODE = "CONFIG_MODE";
const string PC_SET_VALUE_PAL_CHAN_DIVPAM_ID = "Set PAL.CHAN.DIVPAM_ID";
const string PC_SET_VALUE_CONFIG_MODE = "Set CONFIG.MODE";
const string PC_SET_DIVPAM_PANEL = "vision/pcDevices/FGC2_PS_MPC/pcSet_PAL_CHAN_DIVPAM_ID.pnl";
const string PC_SET_CONFIGMODE_PANEL = "vision/pcDevices/FGC2_PS_MPC/pcSet_CONFIG_MODE.pnl";

// constant for import of FGC3_PC
const unsigned PC_CONFIG_FGC3_PC_LENGTH = 12;
const unsigned PC_CONFIG_FGC3_PC_I_FORMAT	= 1;
const unsigned PC_CONFIG_FGC3_PC_V_FORMAT	= 2;
const unsigned PC_CONFIG_FGC3_PC_MIN_I	= 3;
const unsigned PC_CONFIG_FGC3_PC_MAX_I	= 4;
const unsigned PC_CONFIG_FGC3_PC_MIN_V	= 5;
const unsigned PC_CONFIG_FGC3_PC_MAX_V	= 6;
const unsigned PC_CONFIG_FGC3_PC_DEADBAND	= 7;
const unsigned PC_CONFIG_FGC3_PC_DEADBANDTYPE	= 8;
const unsigned PC_CONFIG_FGC3_PC_ARCHIVEACTIVE	= 9;
const unsigned PC_CONFIG_FGC3_PC_DEVICENAME	= 10;
const unsigned PC_CONFIG_FGC3_PC_CMDADDRESS	= 11;
const unsigned PC_CONFIG_FGC3_PC_DIDT = 12;

const int PC_CONFIG_FGC3_PC_INPUT_DPE = 19;
const int PC_CONFIG_FGC3_PC_OUTPUT_DPE = 3;

const string PC_CONFIG_CMW_FGC3_PC_PROPERTY = "SUB_61:fgc.be";
const string PC_CONFIG_CMW_PSEUDO_FGC3_PC_PROPERTY = "SUB_61:pseudo.dev";

